@@ -17,6 +17,6 @@ from .simulators import sim_analytics as analytics
 
 P = CarbonPair
 
-__version__ = "2.3"
-__date__ = "29/Jan/2022"
+__version__ = "2.3.2"
+__date__ = "12/Feb/2022"
 
