@@ -2,7 +2,7 @@ from .fls import fload, fsave
 from .params import Params
 from .pdread import pdread, pathtime, pathtime_yrs, pathtime_days, pdcols, j
 from .sharedvar import SharedVar
-from .strategy import strategy
+from .strategym import strategy
 from .version import require_version
 from .helpers import listdir
 from .pathgenerator import PathGenerator
@@ -28,7 +28,7 @@ def print_version(require=None, all=False):
     if not require is None:
         require_version(require)
 
-def plt_style(style, alt_style):
+def plt_style(style, alt_style=None):
     """
     calls plt.style.use(style) and as fallback alt_style, all wrapped in try blocks
     """
