@@ -1,8 +1,8 @@
 """
 Carbon helper module - retrieve data from CryptoCompare
 """
-__VERSION__ = "1.1"
-__DATE__ = "27/Jan/2023"
+__VERSION__ = "1.4"
+__DATE__ = "01/Feb/2023"
 
 import os as _os
 import pandas as _pd
@@ -38,11 +38,13 @@ class CryptoCompare():
     def __init__(self, apikeyname=None, apikey=None, raiseonerror=None):
         if raiseonerror is None:
             raiseonerror = self.RAISEONERROR
-        self.raiseonerror = raiseonerror        
+        self.raiseonerror = raiseonerror  
+        if not (isinstance(apikey, str) or apikey is None or apikey is True):
+            raise ValueError("apikey must be a string, None, or True", apikey)  
         if apikey is None:
             if apikeyname is None:
                 apikeyname = self.APIKEYNAME
-            apikey = _os.getenv(self.keyname)
+            apikey = _os.getenv(apikeyname)
             if apikey is None:
                 print(f"Can't find API key {apikeyname} in environment variables.")
                 print(f"Use `export {apikeyname}=<value>` to set it BEFORE you launch Jupyter")
@@ -243,10 +245,18 @@ class CryptoCompare():
         """
         return _pd.to_datetime(ts, unit='s', origin='unix') 
     
-    def query_dailypair(self, fsym=None, tsym=None, e=None, limit=False, toTs=None, aspandas=True):
+    FREQ_DAILY = "day"
+    FD = FREQ_DAILY
+    FREQ_HOURLY = "hour"
+    FH = FREQ_HOURLY
+    FREQ_MINUTELY = "minute"
+    FM = FREQ_MINUTELY
+    FREQS = (FREQ_DAILY, FREQ_HOURLY, FREQ_MINUTELY)
+    def query_freqlypair(self, freq, fsym=None, tsym=None, e=None, limit=False, toTs=None, aspandas=True):
         """
-        endpoint = /data/v2/histoday
+        endpoints = /data/v2/histoday, /data/v2/histohour, /data/v2/histominute
         
+        :freq:     FREQ_DAILY/FD, FREQ_HOURLY/FH, or FREQ_MINUTELY/FM
         :fsym:     cryptocurrency symbol of interest
         :tsym:     currency symbol to convert into
         :e:        exchange to obtain data from
@@ -255,22 +265,34 @@ class CryptoCompare():
                    timestamp format either 1452680400 or pd.Timestamp compatible string 
         
         https://min-api.cryptocompare.com/documentation?key=Historical&cat=dataHistoday
+        https://min-api.cryptocompare.com/documentation?key=Historical&cat=dataHistohour
+        https://min-api.cryptocompare.com/documentation?key=Historical&cat=dataHistominute
         """
-        r = self.query(
-            endpoint="/data/v2/histoday",
-            params = {
+        if not freq in self.FREQS:
+            raise ValueError("Unknow frequency {}. Use the FREQ_XXX constants provided.")
+        endpoint = f"/data/v2/histo{freq}"
+        params = {
                 "fsym":     fsym,
                 "tsym":     tsym if not tsym is None else self.DEFAULT_TSYM,
                 "e":        e,
                 "limit":    limit if not limit is False else self.DEFAULT_LIMIT,
                 "toTs":     toTs,
-            }
-        )
+        }
+        r = self.query(endpoint=endpoint, params=params)
         if r is None: return r
+        r_json = r.json()
+        if r_json.get("Response") == "Error":
+            if self.raiseonerror:
+                raise RuntimeError("Query not successfull", r, r_json, endpoint, params)
+            else:
+                return None
         if not aspandas:
-            return r.json().get("Data")
+            return r_json().get("Data")
         try:   
-            df = _pd.DataFrame.from_records(r.json()["Data"]["Data"])
+            # print("[query_freqlypair]", endpoint, params, r)
+            # print("[query_freqlypair] r", r_json())
+            
+            df = _pd.DataFrame.from_records(r_json["Data"]["Data"])
             df["datetime"] = [self.ts_fromcc(ts) for ts in df["time"]]
             df = df.set_index("datetime")
             del df["conversionType"]
@@ -278,9 +300,22 @@ class CryptoCompare():
             del df["time"]
             df = df[['open', 'close', 'high', 'low', 'volumefrom', 'volumeto']]
             return df
-        except:
-            if self.raiseonerror: raise
+        except RuntimeError as e:
+            if self.raiseonerror: 
+                raise RuntimeError("Error {e}", endpoint, params, r)
             return None
+    
+    def query_dailypair(self, *args, **kwargs):
+        """alias for query_freqlypair(FREQ_DAILY, ...)"""
+        return self.query_freqlypair(self.FREQ_DAILY, *args, **kwargs)
+    
+    def query_hourlypair(self, *args, **kwargs):
+        """alias for query_freqlypair(FREQ_HOURLY, ...)"""
+        return self.query_freqlypair(self.FREQ_HOURLY, *args, **kwargs)
+
+    def query_minutelypair(self, *args, **kwargs):
+        """alias for query_freqlypair(FREQ_MINUTELY, ...)"""
+        return self.query_freqlypair(self.FREQ_MINUTELY, *args, **kwargs)
 
     def ccycodes(self, symonly=True, fn=None):
         """
@@ -337,6 +372,85 @@ class CryptoCompare():
         result = ( filter(c.strip()) for c in result)
         return tuple(result)
 
+    def aggr_query(self, 
+            pairs, 
+            fields=None, 
+            incl_raw=True, 
+            incl_raw_aggr=True, 
+            incl_grand_aggr=True, 
+            freq=None, **kwargs):
+        """
+        gets the data for pairs from the API and converts it into tables 
+        
+        :pairs:             the pairs to download, either comma separeted "ETH/USD, BTC/GBP, ..." 
+                            or as tuple of tuples (("ETH", "USD"), ...)
+        :fields:            the fields for which to create aggredate data frames, either comma separated
+                            or as tuple/list; use FREQ_CLOSE and other FIELD_XXX constants here
+        :incl_raw:          whether to include the individual raw data frames
+        :incl_raw_aggr:     whether to include the aggregate raw data frame
+        :incl_grand_aggr:   whether to include a grand aggregate (with double col name)
+        :freq:              the data frequency [FREQ_DAILY (default), FREQ_HOURLY, FREQ_MINUTELY] 
+        :kwargs:            passed through to `query_freqlypair` (eg `e`, `limit`, `toTs`)
+        :returns:           dict with the results
+        
+        dict structure
+        
+        -gaggr
+            - [data]
+        -aggr
+            -open
+                - [data]
+            -close
+                - [data]
+            ...
+        -rawaggr
+            - [data]
+        -raw
+            - "ETH/USD"
+                - [data]
+            ...
+        """
+        if fields is None:
+            fields = self.FIELD_DEFAULT
+        if isinstance(fields, str):
+            fields = self.unjoin(fields)
+        print("[aggr_query] fields", fields)
+            
+        if isinstance(pairs, str):
+            pairs = tuple( self.pt_from_pair(p) for p in self.unjoin(pairs) )
+        print("[aggr_query] pairs", pairs)
+        
+        if freq is None:
+            freq = self.FREQ_DAILY
+            
+        result = {
+            "gaggr": None,
+            "aggr": None,
+            "rawaggr": None,
+            "raw": None,
+        }
+            
+        print("[aggr_query] Querying for raw table", len(pairs))
+        raw_tables = {
+            (fsym, tsym): self.query_freqlypair(freq, fsym=fsym, tsym=tsym)
+            for fsym, tsym in pairs
+        }
+        df_raw = _pd.concat(raw_tables, axis=1)
+        result_raw = {self.pair_from_pt(p):v for p, v in raw_tables.items()}
+        if incl_raw:
+            result["raw"] = result_raw
+        if incl_raw_aggr:
+            result["rawaggr"] = _pd.concat(result_raw, axis=1)
+        
+        print("[aggr_query] Creating aggregate table")
+        result["aggr"] = {
+            field: self.reformat_raw_df(df_raw, field=field, dblcolnm=incl_grand_aggr) 
+            for field in fields
+        }
+        if incl_grand_aggr:
+            result["gaggr"] = _pd.concat(result["aggr"].values(), axis=1)
+        return result
+    
     @staticmethod
     def pairs_fields_from_df(df):
         """
@@ -351,22 +465,37 @@ class CryptoCompare():
         fields = tuple(set(fields))
         return {"pairs": pairs, "fields": fields}
 
-    DEFAULT_DF_FIELD = "close"
-    def reformat_raw_df(self, df, field=None):
+    FIELD_CLOSE = "close"
+    FIELD_OPEN = "open"
+    FIELD_HIGH = "high"
+    FIELD_LOW = "low"
+    FIELD_DEFAULT = FIELD_CLOSE
+    @classmethod
+    def reformat_raw_df(cls, df, field=None, dblcolnm=False):
         """
         reformats a raw df
         
-        :df:     the raw df, as returned by a concatenation eg of daily_pair calls
-        :field:  the name of the price field to use for the price (default: DEFAULT_DF_FIELD)
+        :df:        the raw df, as returned by a concatenation eg of daily_pair calls
+        :field:     the name of the price field to use for the price
+                    use FIELD_OPEN, FIELD_CLOSE etc; default: FIELD_DEFAULT
+        :dblcolnm:  if True, the colname is (field, pair) instead of pair
+        :returns:   the reformatted data frame
         """
         if field is None:
-            field = self.DEFAULT_DF_FIELD
-        return _pd.concat(
-            [
-                df[(*pair, "open")].rename(f"{pair[0]}/{pair[1]}", inplace=True)
-                for pair in self.pairs_fields_from_df(df)["pairs"]
-            ], axis=1
-        )
+            field = cls.FIELD_DEFAULT
+
+        if dblcolnm:
+            result = (
+                df[(*pair, field)].rename((field, f"{pair[0]}/{pair[1]}"), inplace=True)
+                for pair in cls.pairs_fields_from_df(df)["pairs"]
+            )
+        else:
+            result = (
+                df[(*pair, field)].rename(f"{pair[0]}/{pair[1]}", inplace=True)
+                for pair in cls.pairs_fields_from_df(df)["pairs"]
+            )
+
+        return _pd.concat(list(result), axis=1)
 
     @staticmethod
     def pt_from_pair(pair):
@@ -382,13 +511,19 @@ class CryptoCompare():
         """
         return "/".join(pair_t)
 
-    @staticmethod
-    def coinlist(coins, sep=","):
+    @classmethod
+    def coinlist(cls, coins, sep=",", aspt=False):
         """
         creates a coin list from separated string (does not touch lists)
+
+        :coins:    either a string or a list/tuple
+        :sep:      the separator of the string
+        :aspt:     if True, result returned as pair tuple (using `pt_from_pair`)
+        :returns:  original if not str; otherwise tuple of string or pr
         """
+        f = cls.pt_from_pair if aspt else lambda x: x
         if isinstance(coins, str):
-            return tuple(c.strip() for c in coins.split(sep))
+            return tuple(f(c.strip()) for c in coins.split(sep))
         else:
             return coins
 
