@@ -113,7 +113,7 @@ def test_single_orders():
     assert ro.iloc[0]["lid"] == r['lid']
     ro
     
-    r = Sim._add_replace_single_order("USDC")
+    r = Sim._add_replace_single_order("USDC", p_start=None, p_end=None, amt=None, )
     ro=r["orders"]
     print(f"id={r['id']}, lid={r['lid']}")
     assert ro.iloc[0]["id"] == r['id']
@@ -154,7 +154,7 @@ def test_single_orders():
     
     # Setting up the collection account in USDC
     
-    r = Sim._add_replace_single_order("USDC")
+    r = Sim._add_replace_single_order("USDC", p_start=None, p_end=None, amt=None, )
     coid = r["id"]
     print(f"Collection order id coid={coid}")
     r["orders"]
